-------------------------------------------------------------------------------
-- Project  ELE8304 : Circuits intégrés à très grande échelle
-------------------------------------------------------------------------------
-- File     fetch.vhd
-- Authors  Titouan Luard <luardtitouan@gmail.com> 
--          Yann Roberge <yann.roberge@polymtl.ca> 
-- Lab      GRM - Polytechnique Montreal
-- Date     2021-10-29
-------------------------------------------------------------------------------
-- Brief   
------------------------------------------------------------------------
-------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;


library work;
use work.riscv_pkg.all;

entity fetch is
  generic (RESET_VECTOR : std_logic_vector(XLEN-1 downto 0) := (others => '0'));
  port (
    i_clk :       in std_logic;
    i_rstn :      in std_logic;
    i_stall :     in std_logic;
    i_transfert : in std_logic;
    i_target :    in std_logic_vector(XLEN-1 downto 0);
    i_flush :     in std_logic;
    i_mem :       in std_logic_vector(XLEN-1 downto 0);
    
    o_pc:            out std_logic_vector(9 downto 0);
    o_fetch:         out std_logic_vector(XLEN-1 downto 0));
end entity fetch;

architecture beh of fetch is

signal fetch : std_logic_vector(XLEN-1 downto 0);
signal temp_pc :std_logic_vector(XLEN-1 downto 0);
begin

pc: entity work.riscv_pc
    port MAP (
      i_clk => i_clk,
      i_rstn => i_rstn,
      i_stall => i_stall,
      i_transfert => i_transfert,
      i_target => i_target,
      o_pc => temp_pc);

o_pc <= temp_pc(9 downto 0);

fetching: 
process (i_clk, i_rstn) is
  begin
    if rising_edge(i_clk) and i_flush='1' then
      fetch <= std_logic_vector(RESET_VECTOR);

    elsif rising_edge(i_clk) and i_flush='0' then
      
      fetch <= i_mem;
    elsif rising_edge(i_clk) and i_stall = '1' then
        o_fetch <= o_fetch;
        
    end if ;
          o_fetch <= fetch;

end process fetching;


end architecture beh;
